package cc.smtweb.system.bpm.web.sys.user.user;

import cc.smtweb.framework.core.common.SwMap;
import cc.smtweb.framework.core.db.DbEngine;
import cc.smtweb.framework.core.db.impl.DefaultEntity;
import cc.smtweb.framework.core.db.vo.ModelTable;
import cc.smtweb.framework.core.util.PubUtil;
import cc.smtweb.system.bpm.web.design.form.define.PageDataset;
import cc.smtweb.system.bpm.web.engine.dynPage.DynPageDelHandler;
import cc.smtweb.system.bpm.web.engine.dynPage.DynPageListHandler;
import cc.smtweb.system.bpm.web.engine.dynPage.DynPageLoadHandler;
import cc.smtweb.system.bpm.web.engine.dynPage.DynPageSaveHandler;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;

/**
 * Created by Akmm at 2022/7/1 15:47
 * 区划保存
 */
public class UserHandler {

    static class UserSaveHandler extends DynPageSaveHandler {
        private UserStatu userStatu;
        private List<UserRole> roleList;

        @Override
        protected DefaultEntity readBeanFromPage(PageDataset pageDataSet, SwMap data) {
            if (pageDataSet.name.equals("userPartyCard")) {
                roleList = new ArrayList<>();
                Set<Long> userRoles = data.readLongSet("sup_roles");
                DefaultEntity bean = super.readBeanFromPage(pageDataSet, data);
                if (PubUtil.isEmpty(userRoles)) {
                    return bean;
                }
                userRoles.forEach(k -> {
                    UserRole userRole = new UserRole();
                    userRole.init();
                    userRole.setEntityId(DbEngine.getInstance().nextId());
                    userRole.setSupId(bean.getEntityId());
                    userRole.setRoleId(k);
                    roleList.add(userRole);
                });
            }
            return super.readBeanFromPage(pageDataSet, data);
        }

        @Override
        protected void saveBean(DefaultEntity bean) {
            super.saveBean(bean);
            if (bean instanceof User) {
                userStatu = UserStatuCache.getInstance().get(bean.getEntityId());
                if (userStatu == null || userStatu.isEmpty()) {
                    userStatu = new UserStatu();
                    userStatu.init();
                    userStatu.setEntityId(bean.getEntityId());
                    DbEngine.getInstance().findDao(UserStatu.ENTITY_NAME).insertEntity(userStatu);
                    UserStatuCache.getInstance().put(userStatu);
                }
            }
            if (bean instanceof UserParty) {
                Set<Long> ur = UserRoleCache.getInstance().getIdByUser(((UserParty) bean).getUserId());
                DbEngine.getInstance().findDao(UserRole.ENTITY_NAME).deleteEntity(ur);
                DbEngine.getInstance().findDao(UserRole.ENTITY_NAME).batchInsertEntity(roleList);
                UserRoleCache.getInstance().removeList(ur);
                UserRoleCache.getInstance().putList(roleList);
            }
        }
    }

    static class UserDelHandler extends DynPageDelHandler {

        @Override
        protected void localDel(long id, ModelTable table) {
            super.localDel(id, table);
            if (table.getName().equals(User.ENTITY_NAME)) {
                DbEngine.getInstance().findDao(UserStatu.ENTITY_NAME).deleteEntity(id);
                DbEngine.getInstance().findDao(UserParty.ENTITY_NAME).deleteEntity(UserPartyCache.getInstance().getIdByUser(id));
                DbEngine.getInstance().findDao(UserRole.ENTITY_NAME).deleteEntity(UserRoleCache.getInstance().getIdByUser(id));
            }
            if (table.getName().equals(UserParty.ENTITY_NAME)) {
                DbEngine.getInstance().findDao(UserParty.ENTITY_NAME).deleteEntity(id);
                DbEngine.getInstance().findDao(UserRole.ENTITY_NAME).deleteEntity(UserRoleCache.getInstance().getByS(id));
            }
        }

        @Override
        protected void localDelSuccess(long id, ModelTable table) {
            if (table.getName().equals(User.ENTITY_NAME)) {
                UserStatuCache.getInstance().remove(id);
                UserPartyCache.getInstance().removeList(UserPartyCache.getInstance().getIdByUser(id));
                UserRoleCache.getInstance().removeList(UserRoleCache.getInstance().getIdByUser(id));
            }
            if (table.getName().equals(UserParty.ENTITY_NAME)) {
                UserPartyCache.getInstance().remove(id);
                UserRoleCache.getInstance().removeList(UserRoleCache.getInstance().getByS(id));
            }
        }
    }

    static class UserListHandler extends DynPageListHandler {

        public UserListHandler(long pageId, SwMap filter, PageDataset pageDataSet) {
            super(pageId, filter, pageDataSet);
        }
    }

    static class UserLoadHandler extends DynPageLoadHandler {

        @Override
        public DynPageListHandler getListWorker(SwMap filter, PageDataset pageDataSet) {
            UserListHandler listHandler = new UserListHandler(pageId, filter, pageDataSet);
            listHandler.init(params, us);
            return listHandler;
        }
    }
}
